@@ -1,4 +1,4 @@
-/* $OpenBSD: apps.c,v 1.64 2023/04/22 20:50:26 tb Exp $ */
+/* $OpenBSD: apps.c,v 1.67 2023/11/21 17:56:19 tb Exp $ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  *
@@ -199,75 +199,6 @@ program_name(char *in, char *out, int size)
 	strlcpy(out, p, size);
 }
 
-int
-chopup_args(ARGS *arg, char *buf, int *argc, char **argv[])
-{
-	int num, i;
-	char *p;
-
-	*argc = 0;
-	*argv = NULL;
-
-	if (arg->count == 0) {
-		arg->count = 20;
-		arg->data = reallocarray(NULL, arg->count, sizeof(char *));
-		if (arg->data == NULL)
-			return 0;
-	}
-	for (i = 0; i < arg->count; i++)
-		arg->data[i] = NULL;
-
-	num = 0;
-	p = buf;
-	for (;;) {
-		/* first scan over white space */
-		if (!*p)
-			break;
-		while (*p && ((*p == ' ') || (*p == '\t') || (*p == '\n')))
-			p++;
-		if (!*p)
-			break;
-
-		/* The start of something good :-) */
-		if (num >= arg->count) {
-			char **tmp_p;
-			int tlen = arg->count + 20;
-			tmp_p = reallocarray(arg->data, tlen, sizeof(char *));
-			if (tmp_p == NULL)
-				return 0;
-			arg->data = tmp_p;
-			arg->count = tlen;
-			/* initialize newly allocated data */
-			for (i = num; i < arg->count; i++)
-				arg->data[i] = NULL;
-		}
-		arg->data[num++] = p;
-
-		/* now look for the end of this */
-		if ((*p == '\'') || (*p == '\"')) {	/* scan for closing
-							 * quote */
-			i = *(p++);
-			arg->data[num - 1]++;	/* jump over quote */
-			while (*p && (*p != i))
-				p++;
-			*p = '\0';
-		} else {
-			while (*p && ((*p != ' ') &&
-			    (*p != '\t') && (*p != '\n')))
-				p++;
-
-			if (*p == '\0')
-				p--;
-			else
-				*p = '\0';
-		}
-		p++;
-	}
-	*argc = num;
-	*argv = arg->data;
-	return (1);
-}
-
 int
 dump_cert_text(BIO *out, X509 *x)
 {
@@ -1133,7 +1064,7 @@ load_config(BIO *err, CONF *cnf)
 }
 
 char *
-make_config_name()
+make_config_name(void)
 {
 	const char *t = X509_get_default_cert_area();
 	char *p;
@@ -1785,7 +1716,7 @@ args_verify(char ***pargs, int *pargc, int *badarg, BIO *err,
 		}
 		(*pargs)++;
 	} else if (strcmp(arg, "-purpose") == 0) {
-		X509_PURPOSE *xptmp;
+		const X509_PURPOSE *xptmp;
 		if (!argn)
 			*badarg = 1;
 		else {
