@@ -1,4 +1,4 @@
-/* $OpenBSD: dsa_ameth.c,v 1.42 2023/03/04 21:42:49 tb Exp $ */
+/* $OpenBSD: dsa_ameth.c,v 1.57 2024/01/04 17:01:26 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2006.
  */
@@ -75,31 +75,32 @@
 static int
 dsa_pub_decode(EVP_PKEY *pkey, X509_PUBKEY *pubkey)
 {
-	const unsigned char *p, *pm;
-	int pklen, pmlen;
+	X509_ALGOR *algor;
 	int ptype;
 	const void *pval;
-	const ASN1_STRING *pstr;
-	X509_ALGOR *palg;
-	ASN1_INTEGER *public_key = NULL;
-
+	const ASN1_STRING *astr;
+	const unsigned char *key, *params, *p;
+	int key_len, params_len;
+	ASN1_INTEGER *aint = NULL;
 	DSA *dsa = NULL;
+	int ret = 0;
 
-	if (!X509_PUBKEY_get0_param(NULL, &p, &pklen, &palg, pubkey))
-		return 0;
-	X509_ALGOR_get0(NULL, &ptype, &pval, palg);
+	if (!X509_PUBKEY_get0_param(NULL, &key, &key_len, &algor, pubkey))
+		goto err;
+	X509_ALGOR_get0(NULL, &ptype, &pval, algor);
 
 	if (ptype == V_ASN1_SEQUENCE) {
-		pstr = pval;
-		pm = pstr->data;
-		pmlen = pstr->length;
+		astr = pval;
+		params = astr->data;
+		params_len = astr->length;
 
-		if (!(dsa = d2i_DSAparams(NULL, &pm, pmlen))) {
+		p = params;
+		if ((dsa = d2i_DSAparams(NULL, &p, params_len)) == NULL) {
 			DSAerror(DSA_R_DECODE_ERROR);
 			goto err;
 		}
 	} else if (ptype == V_ASN1_NULL || ptype == V_ASN1_UNDEF) {
-		if (!(dsa = DSA_new())) {
+		if ((dsa = DSA_new()) == NULL) {
 			DSAerror(ERR_R_MALLOC_FAILURE);
 			goto err;
 		}
@@ -108,12 +109,13 @@ dsa_pub_decode(EVP_PKEY *pkey, X509_PUBKEY *pubkey)
 		goto err;
 	}
 
-	if (!(public_key = d2i_ASN1_INTEGER(NULL, &p, pklen))) {
+	p = key;
+	if ((aint = d2i_ASN1_INTEGER(NULL, &p, key_len)) == NULL) {
 		DSAerror(DSA_R_DECODE_ERROR);
 		goto err;
 	}
-
-	if (!(dsa->pub_key = ASN1_INTEGER_to_BN(public_key, NULL))) {
+	BN_free(dsa->pub_key);
+	if ((dsa->pub_key = ASN1_INTEGER_to_BN(aint, NULL)) == NULL) {
 		DSAerror(DSA_R_BN_DECODE_ERROR);
 		goto err;
 	}
@@ -124,140 +126,157 @@ dsa_pub_decode(EVP_PKEY *pkey, X509_PUBKEY *pubkey)
 			goto err;
 	}
 
-	ASN1_INTEGER_free(public_key);
-	EVP_PKEY_assign_DSA(pkey, dsa);
-	return 1;
+	if (!EVP_PKEY_assign_DSA(pkey, dsa))
+		goto err;
+	dsa = NULL;
 
-err:
-	if (public_key)
-		ASN1_INTEGER_free(public_key);
+	ret = 1;
+
+ err:
+	ASN1_INTEGER_free(aint);
 	DSA_free(dsa);
-	return 0;
+
+	return ret;
 }
 
 static int
 dsa_pub_encode(X509_PUBKEY *pk, const EVP_PKEY *pkey)
 {
-	DSA *dsa;
-	ASN1_INTEGER *pubint = NULL;
-	ASN1_STRING *str = NULL;
+	const DSA *dsa = pkey->pkey.dsa;
+	ASN1_STRING *astr = NULL;
 	int ptype = V_ASN1_UNDEF;
-	unsigned char *penc = NULL;
-	int penclen;
+	ASN1_INTEGER *aint = NULL;
+	ASN1_OBJECT *aobj;
+	unsigned char *params = NULL, *key = NULL;
+	int params_len = 0, key_len = 0;
+	int ret = 0;
 
-	dsa = pkey->pkey.dsa;
-	if (pkey->save_parameters && dsa->p && dsa->q && dsa->g) {
-		if ((str = ASN1_STRING_new()) == NULL) {
+	if (pkey->save_parameters > 0 && !EVP_PKEY_missing_parameters(pkey)) {
+		if ((params_len = i2d_DSAparams(dsa, &params)) <= 0) {
 			DSAerror(ERR_R_MALLOC_FAILURE);
+			params_len = 0;
 			goto err;
 		}
-		str->length = i2d_DSAparams(dsa, &str->data);
-		if (str->length <= 0) {
+		if ((astr = ASN1_STRING_new()) == NULL) {
 			DSAerror(ERR_R_MALLOC_FAILURE);
 			goto err;
 		}
+		ASN1_STRING_set0(astr, params, params_len);
+		params = NULL;
+		params_len = 0;
 		ptype = V_ASN1_SEQUENCE;
 	}
 
-	if ((pubint = BN_to_ASN1_INTEGER(dsa->pub_key, NULL)) == NULL) {
+	if ((aint = BN_to_ASN1_INTEGER(dsa->pub_key, NULL)) == NULL) {
 		DSAerror(ERR_R_MALLOC_FAILURE);
 		goto err;
 	}
-
-	penclen = i2d_ASN1_INTEGER(pubint, &penc);
-	ASN1_INTEGER_free(pubint);
-
-	if (penclen <= 0) {
+	if ((key_len = i2d_ASN1_INTEGER(aint, &key)) <= 0) {
 		DSAerror(ERR_R_MALLOC_FAILURE);
+		key_len = 0;
 		goto err;
 	}
 
-	if (X509_PUBKEY_set0_param(pk, OBJ_nid2obj(EVP_PKEY_DSA), ptype, str,
-	    penc, penclen))
-		return 1;
+	if ((aobj = OBJ_nid2obj(EVP_PKEY_DSA)) == NULL)
+		goto err;
+	if (!X509_PUBKEY_set0_param(pk, aobj, ptype, astr, key, key_len))
+		goto err;
+	astr = NULL;
+	key = NULL;
+	key_len = 0;
+
+	ret = 1;
 
  err:
-	free(penc);
-	ASN1_STRING_free(str);
+	ASN1_STRING_free(astr);
+	ASN1_INTEGER_free(aint);
+	freezero(params, params_len);
+	freezero(key, key_len);
 
-	return 0;
+	return ret;
 }
 
-/* In PKCS#8 DSA: you just get a private key integer and parameters in the
+/*
+ * In PKCS#8 DSA: you just get a private key integer and parameters in the
  * AlgorithmIdentifier the pubkey must be recalculated.
  */
 static int
 dsa_priv_decode(EVP_PKEY *pkey, const PKCS8_PRIV_KEY_INFO *p8)
 {
-	const unsigned char *p, *pm;
-	int pklen, pmlen;
+	const X509_ALGOR *algor;
 	int ptype;
 	const void *pval;
-	const ASN1_STRING *pstr;
-	const X509_ALGOR *palg;
-	ASN1_INTEGER *privkey = NULL;
+	const ASN1_STRING *astr;
+	const unsigned char *key, *params, *p;
+	int key_len, params_len;
+	ASN1_INTEGER *aint = NULL;
 	BN_CTX *ctx = NULL;
 	DSA *dsa = NULL;
 	int ret = 0;
 
-	if (!PKCS8_pkey_get0(NULL, &p, &pklen, &palg, p8))
-		return 0;
-	X509_ALGOR_get0(NULL, &ptype, &pval, palg);
-	if (ptype != V_ASN1_SEQUENCE)
-		goto decerr;
+	if (!PKCS8_pkey_get0(NULL, &key, &key_len, &algor, p8))
+		goto err;
+	X509_ALGOR_get0(NULL, &ptype, &pval, algor);
 
-	if ((privkey = d2i_ASN1_INTEGER(NULL, &p, pklen)) == NULL)
-		goto decerr;
-	if (privkey->type == V_ASN1_NEG_INTEGER)
-		goto decerr;
+	if (ptype != V_ASN1_SEQUENCE) {
+		DSAerror(DSA_R_PARAMETER_ENCODING_ERROR);
+		goto err;
+	}
 
-	pstr = pval;
-	pm = pstr->data;
-	pmlen = pstr->length;
-	if (!(dsa = d2i_DSAparams(NULL, &pm, pmlen)))
-		goto decerr;
-	/* We have parameters now set private key */
-	if (!(dsa->priv_key = ASN1_INTEGER_to_BN(privkey, NULL))) {
-		DSAerror(DSA_R_BN_ERROR);
-		goto dsaerr;
+	astr = pval;
+	params = astr->data;
+	params_len = astr->length;
+
+	p = params;
+	if ((dsa = d2i_DSAparams(NULL, &p, params_len)) == NULL) {
+		DSAerror(DSA_R_DECODE_ERROR);
+		goto err;
+	}
+	p = key;
+	if ((aint = d2i_ASN1_INTEGER(NULL, &p, key_len)) == NULL) {
+		DSAerror(DSA_R_DECODE_ERROR);
+		goto err;
+	}
+	BN_free(dsa->priv_key);
+	if ((dsa->priv_key = ASN1_INTEGER_to_BN(aint, NULL)) == NULL) {
+		DSAerror(DSA_R_BN_DECODE_ERROR);
+		goto err;
 	}
 
 	/* Check the key for basic consistency before doing expensive things. */
 	if (!dsa_check_key(dsa))
-		goto dsaerr;
+		goto err;
 
 	/* Calculate public key */
-	if (!(dsa->pub_key = BN_new())) {
+	BN_free(dsa->pub_key);
+	if ((dsa->pub_key = BN_new()) == NULL) {
 		DSAerror(ERR_R_MALLOC_FAILURE);
-		goto dsaerr;
+		goto err;
 	}
 
 	if ((ctx = BN_CTX_new()) == NULL) {
 		DSAerror(ERR_R_MALLOC_FAILURE);
-		goto dsaerr;
+		goto err;
 	}
 
 	BN_CTX_start(ctx);
 
 	if (!BN_mod_exp_ct(dsa->pub_key, dsa->g, dsa->priv_key, dsa->p, ctx)) {
 		DSAerror(DSA_R_BN_ERROR);
-		goto dsaerr;
+		goto err;
 	}
 
 	if (!EVP_PKEY_assign_DSA(pkey, dsa))
-		goto decerr;
+		goto err;
+	dsa = NULL;
 
 	ret = 1;
-	goto done;
 
-decerr:
-	DSAerror(DSA_R_DECODE_ERROR);
-dsaerr:
+ err:
 	DSA_free(dsa);
-done:
 	BN_CTX_end(ctx);
 	BN_CTX_free(ctx);
-	ASN1_INTEGER_free(privkey);
+	ASN1_INTEGER_free(aint);
 
 	return ret;
 }
@@ -265,51 +284,59 @@ done:
 static int
 dsa_priv_encode(PKCS8_PRIV_KEY_INFO *p8, const EVP_PKEY *pkey)
 {
-	ASN1_STRING *params = NULL;
-	ASN1_INTEGER *prkey = NULL;
-	unsigned char *dp = NULL;
-	int dplen;
+	const DSA *dsa = pkey->pkey.dsa;
+	ASN1_STRING *astr = NULL;
+	int ptype = V_ASN1_SEQUENCE;
+	ASN1_INTEGER *aint = NULL;
+	ASN1_OBJECT *aobj;
+	unsigned char *params = NULL, *key = NULL;
+	int params_len = 0, key_len = 0;
+	int ret = 0;
 
-	params = ASN1_STRING_new();
-	if (!params) {
+	if ((params_len = i2d_DSAparams(dsa, &params)) <= 0) {
+		DSAerror(ERR_R_MALLOC_FAILURE);
+		params_len = 0;
+		goto err;
+	}
+	if ((astr = ASN1_STRING_type_new(V_ASN1_SEQUENCE)) == NULL) {
 		DSAerror(ERR_R_MALLOC_FAILURE);
 		goto err;
 	}
+	ASN1_STRING_set0(astr, params, params_len);
+	params = NULL;
+	params_len = 0;
 
-	params->length = i2d_DSAparams(pkey->pkey.dsa, &params->data);
-	if (params->length <= 0) {
-		DSAerror(ERR_R_MALLOC_FAILURE);
-		goto err;
-	}
-	params->type = V_ASN1_SEQUENCE;
-
-	/* Get private key into integer */
-	prkey = BN_to_ASN1_INTEGER(pkey->pkey.dsa->priv_key, NULL);
-	if (!prkey) {
+	if ((aint = BN_to_ASN1_INTEGER(dsa->priv_key, NULL)) == NULL) {
 		DSAerror(DSA_R_BN_ERROR);
 		goto err;
 	}
-
-	dplen = i2d_ASN1_INTEGER(prkey, &dp);
-
-	ASN1_INTEGER_free(prkey);
-	prkey = NULL;
-
-	if (!PKCS8_pkey_set0(p8, OBJ_nid2obj(NID_dsa), 0, V_ASN1_SEQUENCE,
-	    params, dp, dplen))
+	if ((key_len = i2d_ASN1_INTEGER(aint, &key)) <= 0) {
+		DSAerror(ERR_R_MALLOC_FAILURE);
+		key_len = 0;
 		goto err;
+	}
 
-	return 1;
+	if ((aobj = OBJ_nid2obj(NID_dsa)) == NULL)
+		goto err;
+	if (!PKCS8_pkey_set0(p8, aobj, 0, ptype, astr, key, key_len))
+		goto err;
+	astr = NULL;
+	key = NULL;
+	key_len = 0;
 
-err:
-	free(dp);
-	ASN1_STRING_free(params);
-	ASN1_INTEGER_free(prkey);
-	return 0;
+	ret = 1;
+
+ err:
+	ASN1_STRING_free(astr);
+	ASN1_INTEGER_free(aint);
+	freezero(params, params_len);
+	freezero(key, key_len);
+
+	return ret;
 }
 
 static int
-int_dsa_size(const EVP_PKEY *pkey)
+dsa_size(const EVP_PKEY *pkey)
 {
 	return DSA_size(pkey->pkey.dsa);
 }
@@ -329,12 +356,9 @@ dsa_security_bits(const EVP_PKEY *pkey)
 static int
 dsa_missing_parameters(const EVP_PKEY *pkey)
 {
-	DSA *dsa;
+	const DSA *dsa = pkey->pkey.dsa;
 
-	dsa = pkey->pkey.dsa;
-	if (dsa->p == NULL || dsa->q == NULL || dsa->g == NULL)
-		return 1;
-	return 0;
+	return dsa->p == NULL || dsa->q == NULL || dsa->g == NULL;
 }
 
 static int
@@ -380,30 +404,17 @@ dsa_pub_cmp(const EVP_PKEY *a, const EVP_PKEY *b)
 }
 
 static void
-int_dsa_free(EVP_PKEY *pkey)
+dsa_free(EVP_PKEY *pkey)
 {
 	DSA_free(pkey->pkey.dsa);
 }
 
-static void
-update_buflen(const BIGNUM *b, size_t *pbuflen)
-{
-	size_t i;
-
-	if (!b)
-		return;
-	if (*pbuflen < (i = (size_t)BN_num_bytes(b)))
-		*pbuflen = i;
-}
-
 static int
 do_dsa_print(BIO *bp, const DSA *x, int off, int ptype)
 {
-	unsigned char *m = NULL;
-	int ret = 0;
-	size_t buf_len = 0;
 	const char *ktype = NULL;
 	const BIGNUM *priv_key, *pub_key;
+	int ret = 0;
 
 	if (ptype == 2)
 		priv_key = x->priv_key;
@@ -422,18 +433,6 @@ do_dsa_print(BIO *bp, const DSA *x, int off, int ptype)
 	else
 		ktype = "DSA-Parameters";
 
-	update_buflen(x->p, &buf_len);
-	update_buflen(x->q, &buf_len);
-	update_buflen(x->g, &buf_len);
-	update_buflen(priv_key, &buf_len);
-	update_buflen(pub_key, &buf_len);
-
-	m = malloc(buf_len + 10);
-	if (m == NULL) {
-		DSAerror(ERR_R_MALLOC_FAILURE);
-		goto err;
-	}
-
 	if (priv_key) {
 		if (!BIO_indent(bp, off, 128))
 			goto err;
@@ -442,43 +441,51 @@ do_dsa_print(BIO *bp, const DSA *x, int off, int ptype)
 			goto err;
 	}
 
-	if (!ASN1_bn_print(bp, "priv:", priv_key, m, off))
+	if (!bn_printf(bp, priv_key, off, "priv:"))
 		goto err;
-	if (!ASN1_bn_print(bp, "pub: ", pub_key, m, off))
+	if (!bn_printf(bp, pub_key, off, "pub: "))
 		goto err;
-	if (!ASN1_bn_print(bp, "P:   ", x->p, m, off))
+	if (!bn_printf(bp, x->p, off, "P:   "))
 		goto err;
-	if (!ASN1_bn_print(bp, "Q:   ", x->q, m, off))
+	if (!bn_printf(bp, x->q, off, "Q:   "))
 		goto err;
-	if (!ASN1_bn_print(bp, "G:   ", x->g, m, off))
+	if (!bn_printf(bp, x->g, off, "G:   "))
 		goto err;
+
 	ret = 1;
-err:
-	free(m);
+
+ err:
 	return ret;
 }
 
 static int
-dsa_param_decode(EVP_PKEY *pkey, const unsigned char **pder, int derlen)
+dsa_param_decode(EVP_PKEY *pkey, const unsigned char **params, int params_len)
 {
-	DSA *dsa;
+	DSA *dsa = NULL;
+	int ret = 0;
 
-	if (!(dsa = d2i_DSAparams(NULL, pder, derlen))) {
+	if ((dsa = d2i_DSAparams(NULL, params, params_len)) == NULL) {
 		DSAerror(ERR_R_DSA_LIB);
-		return 0;
+		goto err;
 	}
-	if (!dsa_check_key(dsa)) {
-		DSA_free(dsa);
-		return 0;
-	}
-	EVP_PKEY_assign_DSA(pkey, dsa);
-	return 1;
+	if (!dsa_check_key(dsa))
+		goto err;
+	if (!EVP_PKEY_assign_DSA(pkey, dsa))
+		goto err;
+	dsa = NULL;
+
+	ret = 1;
+
+ err:
+	DSA_free(dsa);
+
+	return ret;
 }
 
 static int
-dsa_param_encode(const EVP_PKEY *pkey, unsigned char **pder)
+dsa_param_encode(const EVP_PKEY *pkey, unsigned char **params)
 {
-	return i2d_DSAparams(pkey->pkey.dsa, pder);
+	return i2d_DSAparams(pkey->pkey.dsa, params);
 }
 
 static int
@@ -500,13 +507,14 @@ dsa_priv_print(BIO *bp, const EVP_PKEY *pkey, int indent, ASN1_PCTX *ctx)
 }
 
 static int
-old_dsa_priv_decode(EVP_PKEY *pkey, const unsigned char **pder, int derlen)
+old_dsa_priv_decode(EVP_PKEY *pkey, const unsigned char **key, int key_len)
 {
-	DSA *dsa;
+	DSA *dsa = NULL;
 	BN_CTX *ctx = NULL;
 	BIGNUM *result;
+	int ret = 0;
 
-	if ((dsa = d2i_DSAPrivateKey(NULL, pder, derlen)) == NULL) {
+	if ((dsa = d2i_DSAPrivateKey(NULL, key, key_len)) == NULL) {
 		DSAerror(ERR_R_DSA_LIB);
 		goto err;
 	}
@@ -558,23 +566,24 @@ old_dsa_priv_decode(EVP_PKEY *pkey, const unsigned char **pder, int derlen)
 		goto err;
 	}
 
-	BN_CTX_end(ctx);
-	BN_CTX_free(ctx);
+	if (!EVP_PKEY_assign_DSA(pkey, dsa))
+		goto err;
+	dsa = NULL;
 
-	EVP_PKEY_assign_DSA(pkey, dsa);
-	return 1;
+	ret = 1;
 
  err:
 	BN_CTX_end(ctx);
 	BN_CTX_free(ctx);
 	DSA_free(dsa);
-	return 0;
+
+	return ret;
 }
 
 static int
-old_dsa_priv_encode(const EVP_PKEY *pkey, unsigned char **pder)
+old_dsa_priv_encode(const EVP_PKEY *pkey, unsigned char **key)
 {
-	return i2d_DSAPrivateKey(pkey->pkey.dsa, pder);
+	return i2d_DSAPrivateKey(pkey->pkey.dsa, key);
 }
 
 static int
@@ -594,27 +603,16 @@ dsa_sig_print(BIO *bp, const X509_ALGOR *sigalg, const ASN1_STRING *sig,
 	dsa_sig = d2i_DSA_SIG(NULL, &p, sig->length);
 	if (dsa_sig) {
 		int rv = 0;
-		size_t buf_len = 0;
-		unsigned char *m = NULL;
-
-		update_buflen(dsa_sig->r, &buf_len);
-		update_buflen(dsa_sig->s, &buf_len);
-		m = malloc(buf_len + 10);
-		if (m == NULL) {
-			DSAerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
 
 		if (BIO_write(bp, "\n", 1) != 1)
 			goto err;
 
-		if (!ASN1_bn_print(bp, "r:   ", dsa_sig->r, m, indent))
+		if (!bn_printf(bp, dsa_sig->r, indent, "r:   "))
 			goto err;
-		if (!ASN1_bn_print(bp, "s:   ", dsa_sig->s, m, indent))
+		if (!bn_printf(bp, dsa_sig->s, indent, "s:   "))
 			goto err;
 		rv = 1;
-err:
-		free(m);
+ err:
 		DSA_SIG_free(dsa_sig);
 		return rv;
 	}
@@ -676,64 +674,60 @@ dsa_pkey_ctrl(EVP_PKEY *pkey, int op, long arg1, void *arg2)
 	}
 }
 
-/* NB these are sorted in pkey_id order, lowest first */
+const EVP_PKEY_ASN1_METHOD dsa_asn1_meth = {
+	.base_method = &dsa_asn1_meth,
+	.pkey_id = EVP_PKEY_DSA,
 
-const EVP_PKEY_ASN1_METHOD dsa_asn1_meths[] = {
-	{
-		.pkey_id = EVP_PKEY_DSA2,
-		.pkey_base_id = EVP_PKEY_DSA,
-		.pkey_flags = ASN1_PKEY_ALIAS
-	},
+	.pem_str = "DSA",
+	.info = "OpenSSL DSA method",
 
-	{
-		.pkey_id = EVP_PKEY_DSA1,
-		.pkey_base_id = EVP_PKEY_DSA,
-		.pkey_flags = ASN1_PKEY_ALIAS
-	},
+	.pub_decode = dsa_pub_decode,
+	.pub_encode = dsa_pub_encode,
+	.pub_cmp = dsa_pub_cmp,
+	.pub_print = dsa_pub_print,
 
-	{
-		.pkey_id = EVP_PKEY_DSA4,
-		.pkey_base_id = EVP_PKEY_DSA,
-		.pkey_flags = ASN1_PKEY_ALIAS
-	},
+	.priv_decode = dsa_priv_decode,
+	.priv_encode = dsa_priv_encode,
+	.priv_print = dsa_priv_print,
 
-	{
-		.pkey_id = EVP_PKEY_DSA3,
-		.pkey_base_id = EVP_PKEY_DSA,
-		.pkey_flags = ASN1_PKEY_ALIAS
-	},
+	.pkey_size = dsa_size,
+	.pkey_bits = dsa_bits,
+	.pkey_security_bits = dsa_security_bits,
 
-	{
-		.pkey_id = EVP_PKEY_DSA,
-		.pkey_base_id = EVP_PKEY_DSA,
+	.param_decode = dsa_param_decode,
+	.param_encode = dsa_param_encode,
+	.param_missing = dsa_missing_parameters,
+	.param_copy = dsa_copy_parameters,
+	.param_cmp = dsa_cmp_parameters,
+	.param_print = dsa_param_print,
+	.sig_print = dsa_sig_print,
 
-		.pem_str = "DSA",
-		.info = "OpenSSL DSA method",
+	.pkey_free = dsa_free,
+	.pkey_ctrl = dsa_pkey_ctrl,
+	.old_priv_decode = old_dsa_priv_decode,
+	.old_priv_encode = old_dsa_priv_encode
+};
 
-		.pub_decode = dsa_pub_decode,
-		.pub_encode = dsa_pub_encode,
-		.pub_cmp = dsa_pub_cmp,
-		.pub_print = dsa_pub_print,
+const EVP_PKEY_ASN1_METHOD dsa1_asn1_meth = {
+	.base_method = &dsa_asn1_meth,
+	.pkey_id = EVP_PKEY_DSA1,
+	.pkey_flags = ASN1_PKEY_ALIAS,
+};
 
-		.priv_decode = dsa_priv_decode,
-		.priv_encode = dsa_priv_encode,
-		.priv_print = dsa_priv_print,
+const EVP_PKEY_ASN1_METHOD dsa2_asn1_meth = {
+	.base_method = &dsa_asn1_meth,
+	.pkey_id = EVP_PKEY_DSA2,
+	.pkey_flags = ASN1_PKEY_ALIAS,
+};
 
-		.pkey_size = int_dsa_size,
-		.pkey_bits = dsa_bits,
-		.pkey_security_bits = dsa_security_bits,
+const EVP_PKEY_ASN1_METHOD dsa3_asn1_meth = {
+	.base_method = &dsa_asn1_meth,
+	.pkey_id = EVP_PKEY_DSA3,
+	.pkey_flags = ASN1_PKEY_ALIAS,
+};
 
-		.param_decode = dsa_param_decode,
-		.param_encode = dsa_param_encode,
-		.param_missing = dsa_missing_parameters,
-		.param_copy = dsa_copy_parameters,
-		.param_cmp = dsa_cmp_parameters,
-		.param_print = dsa_param_print,
-		.sig_print = dsa_sig_print,
-
-		.pkey_free = int_dsa_free,
-		.pkey_ctrl = dsa_pkey_ctrl,
-		.old_priv_decode = old_dsa_priv_decode,
-		.old_priv_encode = old_dsa_priv_encode
-	}
+const EVP_PKEY_ASN1_METHOD dsa4_asn1_meth = {
+	.base_method = &dsa_asn1_meth,
+	.pkey_id = EVP_PKEY_DSA4,
+	.pkey_flags = ASN1_PKEY_ALIAS,
 };
