@@ -1,4 +1,4 @@
-/* $OpenBSD: mem_dbg.c,v 1.22 2014/06/12 15:49:27 deraadt Exp $ */
+/* $OpenBSD: mem_dbg.c,v 1.27 2024/03/02 11:32:31 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -109,93 +109,10 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <openssl/crypto.h>
-#include <openssl/bio.h>
-#include <openssl/lhash.h>
 
 int
 CRYPTO_mem_ctrl(int mode)
 {
 	return (CRYPTO_MEM_CHECK_OFF);
 }
-
-int
-CRYPTO_is_mem_check_on(void)
-{
-	return (0);
-}
-
-
-void
-CRYPTO_dbg_set_options(long bits)
-{
-	return;
-}
-
-long
-CRYPTO_dbg_get_options(void)
-{
-	return (0);
-}
-
-int
-CRYPTO_push_info_(const char *info, const char *file, int line)
-{
-	return (0);
-}
-
-int
-CRYPTO_pop_info(void)
-{
-	return (0);
-}
-
-int
-CRYPTO_remove_all_info(void)
-{
-	return (0);
-}
-
-void
-CRYPTO_dbg_malloc(void *addr, int num, const char *file, int line,
-    int before_p)
-{
-	/* CRYPTO_dbg_malloc is no longer permitted */
-	abort();
-}
-
-void
-CRYPTO_dbg_free(void *addr, int before_p)
-{
-	/* CRYPTO_dbg_free is no longer permitted */
-	abort();
-}
-
-void
-CRYPTO_dbg_realloc(void *addr1, void *addr2, int num,
-    const char *file, int line, int before_p)
-{
-	/* CRYPTO_dbg_realloc is no longer permitted */
-	abort();
-}
-
-void
-CRYPTO_mem_leaks(BIO *b)
-{
-	return;
-}
-
-void
-CRYPTO_mem_leaks_fp(FILE *fp)
-{
-	return;
-}
-
-
-void
-CRYPTO_mem_leaks_cb(CRYPTO_MEM_LEAK_CB *cb)
-{
-	return;
-}
