@@ -1,4 +1,4 @@
-/* $OpenBSD: ocsp_ht.c,v 1.24 2017/01/29 17:49:23 beck Exp $ */
+/* $OpenBSD: ocsp_ht.c,v 1.27 2023/11/28 09:29:20 jsg Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2006.
  */
@@ -115,6 +115,7 @@ OCSP_REQ_CTX_free(OCSP_REQ_CTX *rctx)
 	free(rctx->iobuf);
 	free(rctx);
 }
+LCRYPTO_ALIAS(OCSP_REQ_CTX_free);
 
 int
 OCSP_REQ_CTX_set1_req(OCSP_REQ_CTX *rctx, OCSP_REQUEST *req)
@@ -128,6 +129,7 @@ OCSP_REQ_CTX_set1_req(OCSP_REQ_CTX *rctx, OCSP_REQUEST *req)
 	rctx->asn1_len = BIO_get_mem_data(rctx->mem, NULL);
 	return 1;
 }
+LCRYPTO_ALIAS(OCSP_REQ_CTX_set1_req);
 
 int
 OCSP_REQ_CTX_add1_header(OCSP_REQ_CTX *rctx, const char *name,
@@ -147,9 +149,10 @@ OCSP_REQ_CTX_add1_header(OCSP_REQ_CTX *rctx, const char *name,
 		return 0;
 	return 1;
 }
+LCRYPTO_ALIAS(OCSP_REQ_CTX_add1_header);
 
 OCSP_REQ_CTX *
-OCSP_sendreq_new(BIO *io, char *path, OCSP_REQUEST *req, int maxline)
+OCSP_sendreq_new(BIO *io, const char *path, OCSP_REQUEST *req, int maxline)
 {
 	OCSP_REQ_CTX *rctx;
 
@@ -192,6 +195,7 @@ OCSP_sendreq_new(BIO *io, char *path, OCSP_REQUEST *req, int maxline)
 
 	return rctx;
 }
+LCRYPTO_ALIAS(OCSP_sendreq_new);
 
 /* Parse the HTTP response. This will look like this:
  * "HTTP/1.0 200 OK". We need to obtain the numeric code and
@@ -367,7 +371,7 @@ next_line:
 
 			rctx->state = OHS_ASN1_HEADER;
 		}
-		/* FALLTRHOUGH */
+		/* FALLTHROUGH */
 
 	case OHS_ASN1_HEADER:
 		/* Now reading ASN1 header: can read at least 2 bytes which
@@ -437,10 +441,11 @@ next_line:
 
 	return 0;
 }
+LCRYPTO_ALIAS(OCSP_sendreq_nbio);
 
 /* Blocking OCSP request handler: now a special case of non-blocking I/O */
 OCSP_RESPONSE *
-OCSP_sendreq_bio(BIO *b, char *path, OCSP_REQUEST *req)
+OCSP_sendreq_bio(BIO *b, const char *path, OCSP_REQUEST *req)
 {
 	OCSP_RESPONSE *resp = NULL;
 	OCSP_REQ_CTX *ctx;
@@ -461,3 +466,4 @@ OCSP_sendreq_bio(BIO *b, char *path, OCSP_REQUEST *req)
 
 	return NULL;
 }
+LCRYPTO_ALIAS(OCSP_sendreq_bio);
