@@ -1,4 +1,4 @@
-/* $OpenBSD: by_file.c,v 1.25 2021/11/10 13:57:42 schwarze Exp $ */
+/* $OpenBSD: by_file.c,v 1.30 2023/12/25 22:14:23 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -66,7 +66,7 @@
 #include <openssl/pem.h>
 #include <openssl/x509.h>
 
-#include "x509_lcl.h"
+#include "x509_local.h"
 
 static int by_file_ctrl(X509_LOOKUP *ctx, int cmd, const char *argc,
     long argl, char **ret);
@@ -75,13 +75,8 @@ static X509_LOOKUP_METHOD x509_file_lookup = {
 	.name = "Load file into cache",
 	.new_item = NULL,
 	.free = NULL,
-	.init = NULL,
-	.shutdown = NULL,
 	.ctrl = by_file_ctrl,
 	.get_by_subject = NULL,
-	.get_by_issuer_serial = NULL,
-	.get_by_fingerprint = NULL,
-	.get_by_alias = NULL,
 };
 
 X509_LOOKUP_METHOD *
@@ -89,33 +84,28 @@ X509_LOOKUP_file(void)
 {
 	return &x509_file_lookup;
 }
+LCRYPTO_ALIAS(X509_LOOKUP_file);
 
 static int
 by_file_ctrl(X509_LOOKUP *ctx, int cmd, const char *argp, long argl,
     char **ret)
 {
-	int ok = 0;
+	const char *file = argp;
+	int type = argl;
 
-	switch (cmd) {
-	case X509_L_FILE_LOAD:
-		if (argl == X509_FILETYPE_DEFAULT) {
-			ok = (X509_load_cert_crl_file(ctx,
-			    X509_get_default_cert_file(),
-			    X509_FILETYPE_PEM) != 0);
-			if (!ok) {
-				X509error(X509_R_LOADING_DEFAULTS);
-			}
-		} else {
-			if (argl == X509_FILETYPE_PEM)
-				ok = (X509_load_cert_crl_file(ctx, argp,
-				    X509_FILETYPE_PEM) != 0);
-			else
-				ok = (X509_load_cert_file(ctx,
-				    argp, (int)argl) != 0);
-		}
-		break;
+	if (cmd != X509_L_FILE_LOAD)
+		return 0;
+
+	if (argl == X509_FILETYPE_DEFAULT) {
+		file = X509_get_default_cert_file();
+		type = X509_FILETYPE_PEM;
 	}
-	return ok;
+	if (X509_load_cert_crl_file(ctx, file, type) != 0)
+		return 1;
+	if (argl == X509_FILETYPE_DEFAULT)
+		X509error(X509_R_LOADING_DEFAULTS);
+
+	return 0;
 }
 
 int
@@ -173,6 +163,7 @@ err:
 	BIO_free(in);
 	return ret;
 }
+LCRYPTO_ALIAS(X509_load_cert_file);
 
 int
 X509_load_crl_file(X509_LOOKUP *ctx, const char *file, int type)
@@ -229,6 +220,7 @@ err:
 	BIO_free(in);
 	return ret;
 }
+LCRYPTO_ALIAS(X509_load_crl_file);
 
 int
 X509_load_cert_crl_file(X509_LOOKUP *ctx, const char *file, int type)
@@ -267,4 +259,4 @@ X509_load_cert_crl_file(X509_LOOKUP *ctx, const char *file, int type)
 	sk_X509_INFO_pop_free(inf, X509_INFO_free);
 	return count;
 }
-
+LCRYPTO_ALIAS(X509_load_cert_crl_file);
