@@ -1,4 +1,4 @@
-/*	$OpenBSD: x509_asid.c,v 1.39 2023/02/16 08:38:17 tb Exp $ */
+/*	$OpenBSD: x509_asid.c,v 1.43 2024/02/20 14:58:16 tb Exp $ */
 /*
  * Contributed to the OpenSSL Project by the American Registry for
  * Internet Numbers ("ARIN").
@@ -70,7 +70,6 @@
 #include <openssl/conf.h>
 #include <openssl/err.h>
 #include <openssl/x509.h>
-#include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
 #include "x509_local.h"
@@ -407,8 +406,12 @@ int
 X509v3_asid_add_inherit(ASIdentifiers *asid, int which)
 {
 	ASIdentifierChoice **choice;
+	ASIdentifierChoice *aic = NULL;
+	int ret = 0;
+
 	if (asid == NULL)
-		return 0;
+		goto err;
+
 	switch (which) {
 	case V3_ASID_ASNUM:
 		choice = &asid->asnum;
@@ -417,19 +420,76 @@ X509v3_asid_add_inherit(ASIdentifiers *asid, int which)
 		choice = &asid->rdi;
 		break;
 	default:
-		return 0;
+		goto err;
 	}
-	if (*choice == NULL) {
-		if ((*choice = ASIdentifierChoice_new()) == NULL)
-			return 0;
-		if (((*choice)->u.inherit = ASN1_NULL_new()) == NULL)
-			return 0;
-		(*choice)->type = ASIdentifierChoice_inherit;
+
+	if (*choice != NULL) {
+		if ((*choice)->type != ASIdentifierChoice_inherit)
+			goto err;
+	} else {
+		if ((aic = ASIdentifierChoice_new()) == NULL)
+			goto err;
+		if ((aic->u.inherit = ASN1_NULL_new()) == NULL)
+			goto err;
+		aic->type = ASIdentifierChoice_inherit;
+
+		*choice = aic;
+		aic = NULL;
 	}
-	return (*choice)->type == ASIdentifierChoice_inherit;
+
+	ret = 1;
+
+ err:
+	ASIdentifierChoice_free(aic);
+
+	return ret;
 }
 LCRYPTO_ALIAS(X509v3_asid_add_inherit);
 
+static int
+ASIdOrRanges_add_id_or_range(ASIdOrRanges *aors, ASN1_INTEGER *min,
+    ASN1_INTEGER *max)
+{
+	ASIdOrRange *aor = NULL;
+	ASRange *asr = NULL;
+	int ret = 0;
+
+	/* Preallocate since we must not fail after sk_ASIdOrRange_push(). */
+	if (max != NULL) {
+		if ((asr = ASRange_new()) == NULL)
+			goto err;
+	}
+
+	if ((aor = ASIdOrRange_new()) == NULL)
+		goto err;
+	if (sk_ASIdOrRange_push(aors, aor) <= 0)
+		goto err;
+
+	if (max == NULL) {
+		aor->type = ASIdOrRange_id;
+		aor->u.id = min;
+	} else {
+		ASN1_INTEGER_free(asr->min);
+		asr->min = min;
+		ASN1_INTEGER_free(asr->max);
+		asr->max = max;
+
+		aor->type = ASIdOrRange_range;
+		aor->u.range = asr;
+		asr = NULL;
+	}
+
+	aor = NULL;
+
+	ret = 1;
+
+ err:
+	ASIdOrRange_free(aor);
+	ASRange_free(asr);
+
+	return ret;
+}
+
 /*
  * Add an ID or range to an ASIdentifierChoice.
  */
@@ -438,9 +498,12 @@ X509v3_asid_add_id_or_range(ASIdentifiers *asid, int which, ASN1_INTEGER *min,
     ASN1_INTEGER *max)
 {
 	ASIdentifierChoice **choice;
-	ASIdOrRange *aor;
+	ASIdentifierChoice *aic = NULL, *new_aic = NULL;
+	int ret = 0;
+
 	if (asid == NULL)
-		return 0;
+		goto err;
+
 	switch (which) {
 	case V3_ASID_ASNUM:
 		choice = &asid->asnum;
@@ -449,39 +512,33 @@ X509v3_asid_add_id_or_range(ASIdentifiers *asid, int which, ASN1_INTEGER *min,
 		choice = &asid->rdi;
 		break;
 	default:
-		return 0;
-	}
-	if (*choice != NULL && (*choice)->type == ASIdentifierChoice_inherit)
-		return 0;
-	if (*choice == NULL) {
-		if ((*choice = ASIdentifierChoice_new()) == NULL)
-			return 0;
-		(*choice)->u.asIdsOrRanges = sk_ASIdOrRange_new(ASIdOrRange_cmp);
-		if ((*choice)->u.asIdsOrRanges == NULL)
-			return 0;
-		(*choice)->type = ASIdentifierChoice_asIdsOrRanges;
-	}
-	if ((aor = ASIdOrRange_new()) == NULL)
-		return 0;
-	if (max == NULL) {
-		aor->type = ASIdOrRange_id;
-		aor->u.id = min;
-	} else {
-		aor->type = ASIdOrRange_range;
-		if ((aor->u.range = ASRange_new()) == NULL)
-			goto err;
-		ASN1_INTEGER_free(aor->u.range->min);
-		aor->u.range->min = min;
-		ASN1_INTEGER_free(aor->u.range->max);
-		aor->u.range->max = max;
-	}
-	if (!(sk_ASIdOrRange_push((*choice)->u.asIdsOrRanges, aor)))
 		goto err;
-	return 1;
+	}
+
+	if ((aic = *choice) != NULL) {
+		if (aic->type != ASIdentifierChoice_asIdsOrRanges)
+			goto err;
+	} else {
+		if ((aic = new_aic = ASIdentifierChoice_new()) == NULL)
+			goto err;
+		aic->u.asIdsOrRanges = sk_ASIdOrRange_new(ASIdOrRange_cmp);
+		if (aic->u.asIdsOrRanges == NULL)
+			goto err;
+		aic->type = ASIdentifierChoice_asIdsOrRanges;
+	}
+
+	if (!ASIdOrRanges_add_id_or_range(aic->u.asIdsOrRanges, min, max))
+		goto err;
+
+	*choice = aic;
+	aic = new_aic = NULL;
+
+	ret = 1;
 
  err:
-	ASIdOrRange_free(aor);
-	return 0;
+	ASIdentifierChoice_free(new_aic);
+
+	return ret;
 }
 LCRYPTO_ALIAS(X509v3_asid_add_id_or_range);
 
@@ -501,6 +558,8 @@ extract_min_max(ASIdOrRange *aor, ASN1_INTEGER **min, ASN1_INTEGER **max)
 		*max = aor->u.range->max;
 		return 1;
 	}
+	*min = NULL;
+	*max = NULL;
 
 	return 0;
 }
@@ -511,6 +570,8 @@ extract_min_max(ASIdOrRange *aor, ASN1_INTEGER **min, ASN1_INTEGER **max)
 static int
 ASIdentifierChoice_is_canonical(ASIdentifierChoice *choice)
 {
+	ASIdOrRange *a, *b;
+	ASN1_INTEGER *a_min = NULL, *a_max = NULL, *b_min = NULL, *b_max = NULL;
 	ASN1_INTEGER *a_max_plus_one = NULL;
 	ASN1_INTEGER *orig;
 	BIGNUM *bn = NULL;
@@ -533,15 +594,8 @@ ASIdentifierChoice_is_canonical(ASIdentifierChoice *choice)
 	 * It's a list, check it.
 	 */
 	for (i = 0; i < sk_ASIdOrRange_num(choice->u.asIdsOrRanges) - 1; i++) {
-		ASIdOrRange *a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
-		    i);
-		ASIdOrRange *b = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
-		    i + 1);
-		ASN1_INTEGER *a_min = NULL,
-		*a_max = NULL,
-		*b_min = NULL,
-		*b_max =
-		    NULL;
+		a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges, i);
+		b = sk_ASIdOrRange_value(choice->u.asIdsOrRanges, i + 1);
 
 		if (!extract_min_max(a, &a_min, &a_max) ||
 		    !extract_min_max(b, &b_min, &b_max))
@@ -583,15 +637,11 @@ ASIdentifierChoice_is_canonical(ASIdentifierChoice *choice)
 	 * Check for inverted range.
 	 */
 	i = sk_ASIdOrRange_num(choice->u.asIdsOrRanges) - 1;
-	{
-		ASIdOrRange *a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
-		    i);
-		ASN1_INTEGER *a_min, *a_max;
-		if (a != NULL && a->type == ASIdOrRange_range) {
-			if (!extract_min_max(a, &a_min, &a_max) ||
-			    ASN1_INTEGER_cmp(a_min, a_max) > 0)
-				goto done;
-		}
+	a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges, i);
+	if (a != NULL && a->type == ASIdOrRange_range) {
+		if (!extract_min_max(a, &a_min, &a_max) ||
+		    ASN1_INTEGER_cmp(a_min, a_max) > 0)
+			goto done;
 	}
 
 	ret = 1;
@@ -620,6 +670,8 @@ LCRYPTO_ALIAS(X509v3_asid_is_canonical);
 static int
 ASIdentifierChoice_canonize(ASIdentifierChoice *choice)
 {
+	ASIdOrRange *a, *b;
+	ASN1_INTEGER *a_min = NULL, *a_max = NULL, *b_min = NULL, *b_max = NULL;
 	ASN1_INTEGER *a_max_plus_one = NULL;
 	ASN1_INTEGER *orig;
 	BIGNUM *bn = NULL;
@@ -650,15 +702,8 @@ ASIdentifierChoice_canonize(ASIdentifierChoice *choice)
 	 * former and fixing the latter.
 	 */
 	for (i = 0; i < sk_ASIdOrRange_num(choice->u.asIdsOrRanges) - 1; i++) {
-		ASIdOrRange *a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
-		    i);
-		ASIdOrRange *b = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
-		    i + 1);
-		ASN1_INTEGER *a_min = NULL,
-		*a_max = NULL,
-		*b_min = NULL,
-		*b_max =
-		    NULL;
+		a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges, i);
+		b = sk_ASIdOrRange_value(choice->u.asIdsOrRanges, i + 1);
 
 		if (!extract_min_max(a, &a_min, &a_max) ||
 		    !extract_min_max(b, &b_min, &b_max))
@@ -743,15 +788,11 @@ ASIdentifierChoice_canonize(ASIdentifierChoice *choice)
 	 * Check for final inverted range.
 	 */
 	i = sk_ASIdOrRange_num(choice->u.asIdsOrRanges) - 1;
-	{
-		ASIdOrRange *a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
-		    i);
-		ASN1_INTEGER *a_min, *a_max;
-		if (a != NULL && a->type == ASIdOrRange_range) {
-			if (!extract_min_max(a, &a_min, &a_max) ||
-			    ASN1_INTEGER_cmp(a_min, a_max) > 0)
-				goto done;
-		}
+	a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges, i);
+	if (a != NULL && a->type == ASIdOrRange_range) {
+		if (!extract_min_max(a, &a_min, &a_max) ||
+		    ASN1_INTEGER_cmp(a_min, a_max) > 0)
+			goto done;
 	}
 
 	/* Paranoia */
