@@ -30,20 +30,25 @@ struct pthread_once {
 };
 typedef struct pthread_once pthread_once_t;
 
+struct _pthread_win32_cb_arg {
+	void (*cb)(void);
+};
+
 static inline BOOL CALLBACK
 _pthread_once_win32_cb(PINIT_ONCE once, PVOID param, PVOID *context)
 {
-	void (*cb) (void) = param;
-	cb();
+	struct _pthread_win32_cb_arg *arg = param;
+	arg->cb();
 	return TRUE;
 }
 
 static inline int
 pthread_once(pthread_once_t *once, void (*cb) (void))
 {
-	BOOL rc = InitOnceExecuteOnce(&once->once, _pthread_once_win32_cb, cb, NULL);
+	struct _pthread_win32_cb_arg arg = { .cb = cb };
+	BOOL rc = InitOnceExecuteOnce(&once->once, _pthread_once_win32_cb, &arg, NULL);
 	if (rc == 0)
-		return -1;
+		return EINVAL;
 	else
 		return 0;
 }
