@@ -1,4 +1,4 @@
-/* $OpenBSD: tls13_internal.h,v 1.96 2022/01/05 17:10:02 jsing Exp $ */
+/* $OpenBSD: tls13_internal.h,v 1.103 2024/01/27 14:31:01 jsing Exp $ */
 /*
  * Copyright (c) 2018 Bob Beck <beck@openbsd.org>
  * Copyright (c) 2018 Theo Buehler <tb@openbsd.org>
@@ -87,13 +87,25 @@ __BEGIN_HIDDEN_DECLS
 #define TLS13_INFO_ACCEPT_EXIT				SSL_CB_ACCEPT_EXIT
 #define TLS13_INFO_CONNECT_EXIT				SSL_CB_CONNECT_EXIT
 
-typedef void (*tls13_alert_cb)(uint8_t _alert_desc, void *_cb_arg);
-typedef ssize_t (*tls13_phh_recv_cb)(void *_cb_arg, CBS *_cbs);
+typedef void (*tls13_alert_cb)(uint8_t _alert_level, uint8_t _alert_desc,
+    void *_cb_arg);
+typedef ssize_t (*tls13_phh_recv_cb)(void *_cb_arg);
 typedef void (*tls13_phh_sent_cb)(void *_cb_arg);
 typedef void (*tls13_handshake_message_cb)(void *_cb_arg);
 typedef void (*tls13_info_cb)(void *_cb_arg, int _state, int _ret);
 typedef int (*tls13_ocsp_status_cb)(void *_cb_arg);
 
+/*
+ * PSK support.
+ */
+
+/*
+ * Known PskKeyExchangeMode values.
+ * https://www.iana.org/assignments/tls-parameters/#tls-pskkeyexchangemode
+ */
+#define TLS13_PSK_KE					0
+#define TLS13_PSK_DHE_KE				1
+
 /*
  * Secrets.
  */
@@ -164,9 +176,19 @@ int tls13_update_server_traffic_secret(struct tls13_secrets *secrets);
 struct tls13_record_layer;
 
 struct tls13_record_layer_callbacks {
+	/* Wire callbacks. */
 	tls_read_cb wire_read;
 	tls_write_cb wire_write;
 	tls_flush_cb wire_flush;
+
+	/* Interceptors. */
+	tls_handshake_read_cb handshake_read;
+	tls_handshake_write_cb handshake_write;
+	tls_traffic_key_cb set_read_traffic_key;
+	tls_traffic_key_cb set_write_traffic_key;
+	tls_alert_send_cb alert_send;
+
+	/* Notification callbacks. */
 	tls13_alert_cb alert_recv;
 	tls13_alert_cb alert_sent;
 	tls13_phh_recv_cb phh_recv;
@@ -176,6 +198,8 @@ struct tls13_record_layer_callbacks {
 struct tls13_record_layer *tls13_record_layer_new(
     const struct tls13_record_layer_callbacks *callbacks, void *cb_arg);
 void tls13_record_layer_free(struct tls13_record_layer *rl);
+void tls13_record_layer_set_callbacks(struct tls13_record_layer *rl,
+    const struct tls13_record_layer_callbacks *callbacks, void *cb_arg);
 void tls13_record_layer_allow_ccs(struct tls13_record_layer *rl, int allow);
 void tls13_record_layer_allow_legacy_alerts(struct tls13_record_layer *rl, int allow);
 void tls13_record_layer_rcontent(struct tls13_record_layer *rl, CBS *cbs);
@@ -188,9 +212,9 @@ void tls13_record_layer_set_legacy_version(struct tls13_record_layer *rl,
 void tls13_record_layer_set_retry_after_phh(struct tls13_record_layer *rl, int retry);
 void tls13_record_layer_handshake_completed(struct tls13_record_layer *rl);
 int tls13_record_layer_set_read_traffic_key(struct tls13_record_layer *rl,
-    struct tls13_secret *read_key);
+    struct tls13_secret *read_key, enum ssl_encryption_level_t read_level);
 int tls13_record_layer_set_write_traffic_key(struct tls13_record_layer *rl,
-    struct tls13_secret *write_key);
+    struct tls13_secret *write_key, enum ssl_encryption_level_t write_level);
 ssize_t tls13_record_layer_send_pending(struct tls13_record_layer *rl);
 ssize_t tls13_record_layer_phh(struct tls13_record_layer *rl, CBS *cbs);
 ssize_t tls13_record_layer_flush(struct tls13_record_layer *rl);
@@ -215,7 +239,6 @@ struct tls13_handshake_msg;
 struct tls13_handshake_msg *tls13_handshake_msg_new(void);
 void tls13_handshake_msg_free(struct tls13_handshake_msg *msg);
 void tls13_handshake_msg_data(struct tls13_handshake_msg *msg, CBS *cbs);
-int tls13_handshake_msg_set_buffer(struct tls13_handshake_msg *msg, CBS *cbs);
 uint8_t tls13_handshake_msg_type(struct tls13_handshake_msg *msg);
 int tls13_handshake_msg_content(struct tls13_handshake_msg *msg, CBS *cbs);
 int tls13_handshake_msg_start(struct tls13_handshake_msg *msg, CBB *body,
@@ -269,6 +292,8 @@ struct tls13_ctx {
 	int phh_count;
 	time_t phh_last_seen;
 
+	tls13_alert_cb alert_sent_cb;
+	tls13_alert_cb alert_recv_cb;
 	tls13_handshake_message_cb handshake_message_sent_cb;
 	tls13_handshake_message_cb handshake_message_recv_cb;
 	tls13_info_cb info_cb;
@@ -281,12 +306,19 @@ struct tls13_ctx {
 #define TLS13_PHH_LIMIT 100
 #endif
 
-struct tls13_ctx *tls13_ctx_new(int mode);
+struct tls13_ctx *tls13_ctx_new(int mode, SSL *ssl);
 void tls13_ctx_free(struct tls13_ctx *ctx);
 
 const EVP_AEAD *tls13_cipher_aead(const SSL_CIPHER *cipher);
 const EVP_MD *tls13_cipher_hash(const SSL_CIPHER *cipher);
 
+void tls13_alert_received_cb(uint8_t alert_level, uint8_t alert_desc, void *arg);
+void tls13_alert_sent_cb(uint8_t alert_level, uint8_t alert_desc, void *arg);
+ssize_t tls13_phh_received_cb(void *cb_arg);
+void tls13_phh_done_cb(void *cb_arg);
+
+int tls13_quic_init(struct tls13_ctx *ctx);
+
 /*
  * Legacy interfaces.
  */
@@ -294,7 +326,6 @@ int tls13_use_legacy_client(struct tls13_ctx *ctx);
 int tls13_use_legacy_server(struct tls13_ctx *ctx);
 int tls13_legacy_accept(SSL *ssl);
 int tls13_legacy_connect(SSL *ssl);
-int tls13_legacy_return_code(SSL *ssl, ssize_t ret);
 ssize_t tls13_legacy_wire_read_cb(void *buf, size_t n, void *arg);
 ssize_t tls13_legacy_wire_write_cb(const void *buf, size_t n, void *arg);
 ssize_t tls13_legacy_wire_flush_cb(void *arg);
